//
//  LARASICFragment.cc   (W.Badgett)
//

#include "LARASICFragment.h"
#include "LariatFragment.h"
#include <stdexcept>

// Constructor
LARASICFragment::LARASICFragment()
{
  configuration.fragmentType = LariatFragment::FRAGMENT_TYPE_LARASIC_CONFIG;
  configuration.fragmentSize = sizeof(LarasicConfiguration_t);
  configuration.eventCounter = 0;
}

uint32_t LARASICFragment::streamToBuffer(std::ostream& output)
{
  uint32_t nBytes=0;

  output.write((char *)&configuration,configuration.fragmentSize);
  nBytes += configuration.fragmentSize;

  return nBytes;
}


uint32_t LARASICFragment::fillFromBuffer(std::istream& input)
{
  uint32_t nBytes=0;

  input.read((char *)&configuration,sizeof(configuration));
  nBytes += sizeof(configuration);

  return nBytes;
}

void LARASICFragment::print(std::ostream& os)
{
  printLarasicConfiguration(os, configuration);
}

void LARASICFragment::printLarasicConfiguration(std::ostream & os, 
						LarasicConfiguration_t config)
{
  os << "LARASICFragment Event " << config.eventCounter << std::endl;
  os << "  ASIC Global   Channels..." << std::endl;
  int ptr=0;
  for ( int i=0; i<N_ASICS; i++)
  {
    char line[256];
    sprintf(line,"  %2d     %2.2X    ", i, config.serialArray[ptr]);
    ptr++;
    os << line;
    for ( int j=0; j<N_CHANNELS; j++)
    {
      sprintf(line," %2.2X", config.serialArray[ptr]);
      os << line;
      ptr++;
    }
    os << std::endl;
  }
}

std::vector<uint32_t> LARASICFragment::findTestChannels(LarasicConfiguration_t config)
{
  std::vector<uint32_t> channels;
  int ptr = 0;
  int channel = 0;
  for ( int i=0; i<N_ASICS; i++)
  {
    ptr++; // skip global header
    for ( int j=0; j<N_CHANNELS; j++)
    {
      if ( testEnable(config.serialArray[ptr]) )
      { channels.push_back(channel);}
      channel++;
      ptr++;
    }
  }
  return(channels);
}

uint8_t LARASICFragment::packChannel(uint8_t gain,
				     uint8_t filter,
				     uint8_t dcOffset,
				     uint8_t testEnable,
				     uint8_t outputCoupling,
				     uint8_t outputBuffer)
{
  return( ( ( gain & SG_M )            << SG_S  ) | 
	  ( ( filter & ST_M )          << ST_S  ) | 
	  ( ( dcOffset & SNC_M )       << SNC_S ) | 
	  ( ( testEnable & STS_M )     << STS_S ) |
	  ( ( outputCoupling & SDC_M ) << SDC_S ) |
	  ( ( outputBuffer & SDF_M )   << SDF_S ) );
}

			   
