//
//  ReadoutError.cc   (W.Badgett)
//

#include "ReadoutError.h"
#include "LariatFragment.h"
#include <stdexcept>

// Constructor
ReadoutError::ReadoutError()
{
  readoutError.fragmentType = LariatFragment::FRAGMENT_TYPE_READOUT_ERROR;
  readoutError.fragmentSize = sizeof(ReadoutError_t);
}

uint32_t ReadoutError::streamToBuffer(std::ostream& output)
{
  uint32_t nBytes=0;

  int headerLength = sizeof(struct ReadoutError) - sizeof(char *);
  output.write((char *)&readoutError,headerLength);
  nBytes += headerLength;

  int textLength = readoutError.fragmentSize - headerLength;
  output.write(readoutError.what, textLength);
  nBytes += textLength;
  return nBytes;
}


uint32_t ReadoutError::fillFromBuffer(std::istream& input)
{
  uint32_t nBytes=0;

  int headerLength = sizeof(struct ReadoutError) - sizeof(char *);
  input.read((char *)&readoutError,headerLength);
  nBytes += headerLength;

  int textLength = readoutError.fragmentSize - headerLength;
  readoutError.what = (char *)malloc(textLength);
  input.read(readoutError.what, textLength);
  nBytes += textLength;
  return nBytes;
}

void ReadoutError::print(std::ostream & os)
{
  os << "ReadoutError" << std::endl;
  os << "  RunNumber     " << readoutError.runNumber  << std::endl;
  os << "  SpillNumber   " << readoutError.spillNumber  << std::endl;
  os << "  TimeStamp     " << readoutError.timeStamp  << std::endl;
  os << "  What          " << readoutError.what << std::endl;
}

